#include "LPD8806_kopterlight.h"
#include "SPI.h"

/*****************************************************************************/
// Kopter light control on LPD8806-based RGB LED Modules in a strip!
/*****************************************************************************/

/*****************************************************************************/
// To be changed according to your setup 
// The LED strips are 40 LEDs per meter but you can extend/cut the strip

#define rigger 6  // number of rigger (default: 4)
#define riggerSize  44  // Pixels / LEDs per rigger  (default: 10)
#define stripSize (riggerSize*rigger)

// define varible for random effect selector
int effect_selector  = 0;

// Use only if Hardware SPI is not possible
// int dataPin = 3;        // Software SPI Data Pin
// int clockPin = 2;       // Software SPI Clock Pin
// Soft SPI
// LPD8806 strip = LPD8806(stripSize, dataPin, clockPin, rigger);

// HW SPI
// on Arduino 168/328 thats data = 11, and clock = pin 13
LPD8806 strip = LPD8806(stripSize, rigger);

/*****************************************************************************/
//Do not change or remove these!
void setup() {
  // Start the Serial output to indicate actual effect and rgb-colour
  Serial.begin(9600);
  // Start up the LED strip
  strip.begin();
  // Update the strip, to start they are all 'off'
  strip.show();
}

/*****************************************************************************/
//                       Space for your light effects:
/*****************************************************************************/
// Only inside Void Loop() you are allowed to design your light effects !!
// ( If you like to program a new pattern or sequences, refer to sequences.ino)
/*****************************************************************************/
// Parameter: Colors are defined by the additive RGB color model. 
//            n = LED No on rigger 
//            r = red    ( max. 128)
//            g = green  ( max. 128)
//            b = blue   ( max. 128)
//            dly = delay ( typical: 10 - 50 ) 
//            cyl = cyles ( how many times this sequence will repeat )
//            riX = rigger to set pixel / to show sequence on x= number of  rigger
/*****************************************************************************/
void loop() {
  
// use analogInput noise for random number calculation
  randomSeed(analogRead(0)); 

// array colours: 0 red, 1 green, 2 blue, 3 yellow, 4 cyan, 5 magenta, 6 white, 7 white, 8 light green, 9 light blue, 10 light red, 11 light cyan, 12 purple, 13 pink, 14 turquoise, 15 Random 
int r_colour_value[] = {127, 0  , 0  , 127, 0  , 127, 127, 63 , 63 , 0  , 127, 63 , 127, 63 , random(0,128) }; // r volour array
int g_colour_value[] = {0  , 127, 0  , 127, 127, 0  , 127, 63 , 127, 63 , 0  , 63 , 63 , 127, random(0,128) }; // g volour array
int b_colour_value[] = {0  , 0  , 127, 0  , 127, 127, 127, 63 , 0  , 127, 63 , 127, 63 , 63 , random(0,128) }; // b volour array

String ColourString ="";
  
  clearstrip ();

// select random or manual effect
// ------------------------------

int effect_selector = random(0, 20); // random effect selection
// int effect_selector = 5; // manual effect selection 0-19

//  random or manual colour selection
// ----------------------------------

int colour_array_index = random(0,15); // choose colour randomly from defined rgb array
// int colour_array_index = 15; // manually select colour from defined rgb array

// start of effect section
// -----------------------

if (effect_selector == 0)
  {
    // Set single LED on selected rigger:
    // setLED(n, r, g, b, ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'single LEDs', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
        
    for (int i=0 ; i<=random(0, riggerSize)/2; i++) // random pixels with given colour
      {
      setLED(random(0, riggerSize), r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1,1,1,1,1,1,1,1);
      strip.show();
      delay(random(0,10));
      }
    clearstrip ();

    for (int i=0 ; i<=random(0, riggerSize)/2; i++) // fade from inside to outside
      {
      setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1,1,1,1,1,1,1,1);
      strip.show();
      delay(random(0,10));
      }
    clearstrip (); 
  
   for (int i=0 ; i<=random(0, riggerSize)/2; i++) // fade from outside to inside
      {
      setLED(riggerSize-i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1,1,1,1,1,1,1,1);
      strip.show();
      delay(random(0,10));
      }
    clearstrip (); 
    
   
    for (int i=0 ; i<=random(0, riggerSize)/2; i++)
          setLED(riggerSize-i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1,1,1,1,1,1,1,1);
    strip.show();
    delay(200);
    clearstrip ();
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
          setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1, 0, 0, 0, 0, 0, 0, 0);
    strip.show();
    delay(100);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 0, 1, 0, 0, 0, 0, 0, 0);
    strip.show();
    delay(100);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 0, 0, 1, 0, 0, 0, 0, 0);
    strip.show();
    delay(100);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 0, 0, 0, 1, 0, 0, 0, 0);
    strip.show();
    delay(100);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(riggerSize-i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1, 0, 1, 0, 0, 0, 0, 0);
    strip.show();
    delay(300);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 0, 1, 0, 1, 0, 0, 0, 0);
    strip.show();
    delay(300);
    
    for (int i=0 ; i<=random(0, riggerSize); i++)
    setLED(i, r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index], 1, 1, 1, 1, 1, 1, 1, 1);
    strip.show();
    delay(100);

  }
  if (effect_selector == 1)
  {
    // Fill the entire strip with...
    // colorWipe(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Fill Stripe', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    int random_r = random(0,128);
    int random_g = random(0,128);
    int random_b = random(0,128);
    
    colorWipe(r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index],30, 1,1,1,1,1,1,1,1);
    
    colorWipe(r_colour_value[colour_array_index], b_colour_value[colour_array_index], random_b                          ,30, 1,0,1,0,1,0,1,0);
    colorWipe(r_colour_value[colour_array_index], random_g                          , g_colour_value[colour_array_index],30, 0,1,0,1,0,1,0,1);
    colorWipe(0                                 , b_colour_value[colour_array_index], g_colour_value[colour_array_index],30, 1,0,1,0,1,0,1,0);
    colorWipe(random_r                          , b_colour_value[colour_array_index], 0                                 ,30, 0,1,0,1,0,1,0,1);
    
    colorWipe(r_colour_value[colour_array_index], g_colour_value[colour_array_index], b_colour_value[colour_array_index],30, 1,1,1,1,1,1,1,1);
    
    
    colorWipe(random_r                          , b_colour_value[colour_array_index], g_colour_value[colour_array_index],30, 1,1,0,0,1,1,0,0);
    colorWipe(r_colour_value[colour_array_index], b_colour_value[colour_array_index], random_b                          ,30, 0,0,1,1,0,0,1,1);    
    
    colorWipe(0                                 , b_colour_value[colour_array_index], random_b                          ,30, 1,0,1,0,1,0,1,0);
    colorWipe(r_colour_value[colour_array_index], random_g                          , 0                                 ,30, 0,1,0,1,0,1,0,1);
    colorWipe(random_r                          , 0                                 , g_colour_value[colour_array_index],30, 1,0,1,0,1,0,1,0);
    
    colorWipe(b_colour_value[colour_array_index], g_colour_value[colour_array_index], r_colour_value[colour_array_index],30, 0,0,1,1,0,0,1,1);
    colorWipe(g_colour_value[colour_array_index], r_colour_value[colour_array_index], b_colour_value[colour_array_index],30, 1,1,0,0,1,1,0,0);
  }

  if (effect_selector == 2)
  {
    // Send a simple pixel chase in...
    // colorChase(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Colorchase', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    colorChase(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,100), 1,1,1,1,1,1,1,1);
  }

  if (effect_selector == 3)
  {
    // Send a simple pixel chase in (reversed)...
    // colorChaseRev(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Colorchase', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    colorChaseRev(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,100), 1,1,1,1,1,1,1,1);
  } 
  
  if (effect_selector == 4)
  {
    // Fade In Effect:
    // fadein(r, g, b, dly, ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Fade In', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    fadein(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(0,50), 1,1,1,1,1,1,1,1);
  }
  
  if (effect_selector == 5)
  {
    // Fade Out Effect:
    // fadeout(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Fade Out', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    fadeout(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(0,50), 1,1,1,1,1,1,1,1);
  }
  
  if (effect_selector == 6)
  {
     // Flashing Light Effect (given colour and complementary colour)
    // flashLight(r,g,b,dly,cyl,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Flashing Light', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i<=random(0, 5); i++)
    flashLight(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(25,100),i, 1,1,1,1,1,1,1,1);
  }
    
  if (effect_selector == 7)
  {
    // Police Light Effect (given colour and complementary colour)
    // police(r,g,b,dly,cyl,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8);
    Serial.print("Effect 'Police Light', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(0, 5); i++)
    police(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(75,120),i, 1,1,1,1,1,1,1,1);
  }
  
  if (effect_selector == 8)
  {
    // Color sparkles
    // dither(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Color Sparkles', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    dither(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,100), 1,0,1,0,1,0,1,0);
    dither(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,100), 0,1,0,1,0,1,0,1);
    dither(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,100), 1,1,1,1,1,1,1,1);
  }
  
  if (effect_selector == 9)
  {
    // Back-and-forth lights
    // scanner(r,g,b,dly,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Scanner', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    scanner(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(25,75), 1,1,1,1,1,1,1,1);
  }
  
  if (effect_selector == 10)
  {
    // Spiral light
    // spiral( r, g, b, wait);
    Serial.print("Effect 'Spiral', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(1, 5); i++)
    spiral(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index], random(1, 75));
    clearstrip ();
  }
  
  if (effect_selector == 11)
  {
    // Reverse Spiral light
    // spiralRev( r, g, b, wait);
    Serial.print("Effect 'Reverse Spiral', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(1, 5); i++)
    spiralRev(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index], random(1, 75));
    clearstrip ();
  }
  
  if (effect_selector == 12)
  {
    // Circling Lights Effect  (given colour and complementary colour)
    // circlinglights(r,g,b,dly,cyl,ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8);
    Serial.print("Effect 'Circling Lights', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(1, 5); i++)
    circlinglights(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(10,50),random(0, 5), 1,1,1,1,1,1,1,1);
  }
   
  if (effect_selector == 13)
  {
    //
    // flashingCircle( r, g, b, wait);
    Serial.print("Effect 'Flashing Circle', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(1, 5); i++){
    flashingCircle(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index], random(1, 50));
    }
    clearstrip ();
    
  }
  
  if (effect_selector == 14)
  {
    //
    // flashingCircleRev( r, g, b, wait);
    Serial.print("Effect 'Reverse Flashing Circle ', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    for (int i=0 ; i <= random(1, 5); i++)
    flashingCircleRev(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index], random(1, 50));
    clearstrip ();
  }
  
  if (effect_selector == 15)
  {
    // Wavy ripple effects
    // wave(r,g,b,dly, stp, ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Wave', Colour RGB ");
    Serial.print(r_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.print(g_colour_value[colour_array_index]);
    Serial.print(" ");
    Serial.println(b_colour_value[colour_array_index]);
    
    wave(r_colour_value[colour_array_index],g_colour_value[colour_array_index],b_colour_value[colour_array_index],random(5,50),random(1,10), 1,1,1,1,1,1,1,1);
    clearstrip (); 
  }
  
  if (effect_selector == 16)
  {
    // make a pretty rainbow cycle!
    // rainbowCycle(dly, cyl, ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Rainbow'");
    
    rainbowCycle(0,random(0, 5), 1,1,1,1,1,1,1,1);
    clearstrip ();
  }

  if (effect_selector == 17)
  {
    // make a pretty rainbow cycle btu using all LEDs at once!
    // rainbowCycleAll(dly, cyl, ri1, ri2, ri3, ri4, ri5, ri6, ri7, ri8)
    Serial.print("Effect 'Rainbow on all LEDs'");
    
    rainbowCycleAll(random(0, 100),random(0, 5), 1,1,1,1,1,1,1,1);
    clearstrip ();
  }  
  
  if (effect_selector == 18)
  {
    // make random colour pixels on all riggers
    Serial.println("Effect 'Random Colour Pixels'");
    for (int j=0; j<=random(20,100); j++)
      {
      for (int i=0 ; i<=random(0, riggerSize)/2; i++) // random pixels with random colour
        {
        setLED(random(0, riggerSize), random(0,128), random(0,128), random(0,128), 1,1,1,1,1,1,1,1);
        strip.show();
        delay(random(0,10));
        }
        
      for (int i=0 ; i<=random(0, riggerSize)/2; i++) // random pixels with random colour
        {
        setLED(random(0, riggerSize), 0            , random(0,128), random(0,128), 1,1,1,1,1,1,1,1);
        strip.show();
        delay(random(0,10));
        }
        
      for (int i=0 ; i<=random(0, riggerSize)/2; i++) // random pixels with random colour
        {
        setLED(random(0, riggerSize), random(0,128), 0            , random(0,128), 1,1,1,1,1,1,1,1);
        strip.show();
        delay(random(0,10));
        }
        
      for (int i=0 ; i<=random(0, riggerSize)/2; i++) // random pixels with random colour
        {
        setLED(random(0             , riggerSize), random(0,128), random(0,128), 0, 1,1,1,1,1,1,1,1);
        strip.show();
        delay(random(0,10));
        }
      }
      clearstrip ();
  }

  if (effect_selector == 19)
  {
    // invert color of rigger 1 (heading forward mode)
    Serial.print("Effect 'Rigger Heading forward Mode'");
    int rigger_colour = random(0,385);
    
    for (int i=0; i < riggerSize; i++) {
      strip.setPixelColor(i, Wheel(rigger_colour)    ,1,0,0,0,0,0,0,0);
      strip.setPixelColor(i, 384-Wheel(rigger_colour),0,1,1,1,1,1,1,1);
      strip.show();
    } 
     delay(1000);
    
    clearstrip ();
  }   
  
//  else
//    Serial.print("Effect does not exist");
//    delay (1000);

}


